"""
Hawk Backtester: A fast portfolio backtesting system.

This package provides a high-performance backtester implemented in Rust
with Python bindings.
"""

from .hawk_backtester import HawkBacktester

__all__ = ["HawkBacktester"]
